"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _RunCounter_runs, _BatchRunCounter_resubmitInfo;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchRunCounter = exports.RunCounter = void 0;
const internal_1 = require("@fluidframework/core-utils/internal");
/**
 * Utility for tracking the number of concurrent runs of a particular operation.
 */
class RunCounter {
    constructor() {
        _RunCounter_runs.set(this, 0);
    }
    get running() {
        return __classPrivateFieldGet(this, _RunCounter_runs, "f") !== 0;
    }
    get runs() {
        return __classPrivateFieldGet(this, _RunCounter_runs, "f");
    }
    run(act) {
        var _a, _b;
        __classPrivateFieldSet(this, _RunCounter_runs, (_a = __classPrivateFieldGet(this, _RunCounter_runs, "f"), _a++, _a), "f");
        try {
            return act();
        }
        finally {
            __classPrivateFieldSet(this, _RunCounter_runs, (_b = __classPrivateFieldGet(this, _RunCounter_runs, "f"), _b--, _b), "f");
        }
    }
}
exports.RunCounter = RunCounter;
_RunCounter_runs = new WeakMap();
/**
 * A specific use case of RunCounter - for when we are accumulating a batch in ContainerRuntime
 */
class BatchRunCounter extends RunCounter {
    constructor() {
        super(...arguments);
        _BatchRunCounter_resubmitInfo.set(this, void 0);
    }
    /**
     * Gets the resubmit info if currently resubmitting a batch, or undefined if not under resubmit.
     */
    get resubmitInfo() {
        return __classPrivateFieldGet(this, _BatchRunCounter_resubmitInfo, "f");
    }
    run(act, resubmitInfo) {
        (0, internal_1.assert)(__classPrivateFieldGet(this, _BatchRunCounter_resubmitInfo, "f") === undefined, "Reentrancy not allowed in BatchRunCounter");
        __classPrivateFieldSet(this, _BatchRunCounter_resubmitInfo, resubmitInfo, "f");
        try {
            return super.run(act);
        }
        finally {
            __classPrivateFieldSet(this, _BatchRunCounter_resubmitInfo, undefined, "f");
        }
    }
}
exports.BatchRunCounter = BatchRunCounter;
_BatchRunCounter_resubmitInfo = new WeakMap();
//# sourceMappingURL=runCounter.js.map